@@ -31,6 +31,7 @@ from functools import partial
 from math import floor
 import time, random, datetime, socket, ifaddr
 
+
 # A couple of constants
 LISTEN_IP = "0.0.0.0"
 UDP_BROADCAST_IP = "255.255.255.255"
@@ -862,6 +863,7 @@ class Light(Device):
         super(Light, self).__init__(loop, mac_addr, ip_addr, port, parent)
         self.color = None
         self.color_zones = None
+        self.zones_count = 1
         self.infrared_brightness = None
         self.hev_cycle = None
         self.hev_cycle_configuration = None
@@ -1195,6 +1197,97 @@ class Light(Device):
                     resp.direction
                 ).name.capitalize()
 
+    def get_extended_color_zones(self, callb=None):
+        """
+        Convenience method to request the state of all zones of a multizone device
+        in a single request.
+
+        The device must have the extended_multizone feature to use this method.
+
+        This method will request the information from the device and request that callb
+        be executed when a response is received.
+
+        :param callb: Callable to be used when the response is received. If not set,
+                    self.resp_set_multizonemultizoneextendedcolorzones will be used.
+        :type callb: callable
+        :returns: None
+        :rtype: None
+        """
+        self.req_with_resp(
+            MultiZoneGetExtendedColorZones,
+            MultiZoneStateExtendedColorZones,
+            callb=callb,
+        )
+
+    def set_extended_color_zones(
+        self,
+        colors,
+        colors_count,
+        zone_index=0,
+        duration=0,
+        apply=1,
+        callb=None,
+        rapid=False,
+    ):
+        """
+        Convenience method to set the state of all zones on a multizone device in
+        a single request.
+
+        The device must have the extended_multizone feature to use this method.
+        There must be 82 color tuples in the colors list regardless of how many
+        zones the device has. Use the colors_count parameter to specify the number
+        of colors from the colors list that should be applied to the device and
+        use the zone_index parameter to specify the starting zone.
+
+        :param colors List of color dictionaries with HSBK keys
+        :type colors List[dict[str, int]]
+        :param colors_count How many color values in the color list to apply to the device
+        :type colors_count int
+        :param zone_index Which zone to start applying the colors from (default 0)
+        :type zone_index int
+        :param duration duration in seconds to apply the colors (default 0)
+        :type duration int
+        :param apply whether to apply the colors or buffer the new value (default 1 or apply)
+        :type apply int
+        :param callb Callback function to invoke when the response is received
+        :type callb Callable
+        :returns None
+        :rtype None
+        """
+        if len(colors) == 82:
+            args = {
+                "duration": duration,
+                "apply": apply,
+                "zone_index": zone_index,
+                "colors_count": colors_count,
+                "colors": colors,
+            }
+            mypartial = partial(self.resp_set_multizoneextendedcolorzones, args=args)
+
+            if callb:
+                mycallb = lambda x, y: (mypartial(y), callb(x, y))
+            else:
+                mycallb = lambda x, y: mypartial(y)
+
+            if rapid:
+                self.fire_and_forget(
+                    MultiZoneSetExtendedColorZones, args, num_repeats=1
+                )
+                mycallb(self, None)
+            else:
+                self.req_with_ack(MultiZoneSetExtendedColorZones, args, callb=mycallb)
+
+    def resp_set_multizoneextendedcolorzones(self, resp, args=None):
+        """Default callback for get_extended_color_zones"""
+        if args:
+            if self.color_zones:
+                for i in range(args["zone_index"], args["colors_count"]):
+                    self.color_zones[i] = args["colors"][i]
+
+        elif resp:
+            self.zones_count = resp.zones_count
+            self.color_zones = resp.colors[resp.zone_index : resp.colors_count]
+
     # value should be a dictionary with the the following keys: transient, color, period, cycles, skew_ratio, waveform
     def set_waveform(self, value, callb=None, rapid=False):
         """Convenience method to animate the light, a dictionary with the the following
@@ -1461,6 +1554,94 @@ class Light(Device):
         if resp:
             self.last_hev_cycle_result = LAST_HEV_CYCLE_RESULT.get(resp.result)
 
+    def get_tile_effect(self, callb=None):
+        """Convenience method to get the currently running effect on a Tile or Candle.
+
+        The value returned is the previously known state of the effect. Use a callback
+        to get the actual current state.
+
+        :param callb: callable to be used when a response is received. If not set,
+                      self.resp_set_tileeffect will be used.
+        :type callb: callable
+        :returns: current effect details as a dictionary
+        :rtype: dict
+        """
+        response = self.req_with_resp(
+            TileGetTileEffect, TileStateTileEffect, callb=callb
+        )
+        return self.effect
+
+    def set_tile_effect(self, effect=0, speed=3, palette=None, callb=None, rapid=False):
+        """Convenience method to start or stop a firmware effect on matrix devices.
+
+        A palette of up to 16 HSBK tuples can be provided for the MORPH effect, otherwise
+        it will use the same Exciting theme used by the LIFX smart phone app.
+
+        :param effect: 0/Off, 2/Morph, 3/Flame
+        :type effect: int/str
+        :param speed: time in seconds for one cycle of the effect to travel the length of the device
+        :type speed: int
+        :param palette: a list of up to 16 HSBK tuples to use for the Morph effect
+        :type palette: list[tuple(hue, saturation, brightness, kelvin)]
+        :param callb: a callback to use when the response is received
+        :type callb: callable
+        :param rapid: whether to request an acknowledgement or not
+        :type rapid: bool
+        :returns: None
+        :rtype: None
+        """
+
+        # Exciting theme
+        default_tile_palette = [
+            (0, 65535, 65535, 3500),
+            (7282, 65535, 65535, 3500),
+            (10923, 65535, 65535, 3500),
+            (22209, 65535, 65535, 3500),
+            (43509, 65535, 65535, 3500),
+            (49334, 65535, 65535, 3500),
+            (53521, 65535, 65535, 3500),
+        ]
+
+        typ = effect
+        if type(effect) == str:
+            typ = TileEffectType[effect.upper()].value
+        elif type(effect) == int:
+            typ = effect if effect in [e.value for e in TileEffectType] else 0
+
+        speed = floor(speed * 1000) if 0 < speed <= 60 else 3000
+        if palette is None:
+            palette = default_tile_palette
+        if len(palette) > 16:
+            palette = palette[:16]
+        palette_count = len(palette)
+
+        payload = {
+            "type": typ,
+            "speed": speed,
+            "duration": 0,
+            "palette_count": palette_count,
+            "palette": palette,
+        }
+        if rapid:
+            self.fire_and_forget(TileSetTileEffect, payload)
+        else:
+            self.req_with_ack(TileSetTileEffect, payload, callb=callb)
+
+    def resp_set_tiletileeffect(self, resp):
+        """Default callback for get_tile_effect and set_tile_effect"""
+        if resp:
+            self.effect = {"effect": TileEffectType(resp.effect).name.upper()}
+
+            if resp.effect != 0:
+                self.effect["speed"] = resp.speed / 1000
+                self.effect["duration"] = (
+                    0.0
+                    if resp.duration == 0
+                    else float(f"{self.effect['duration']/1000000000:4f}")
+                )
+                self.effect["palette_count"] = resp.palette_count
+                self.effect["palette"] = resp.palette
+
     def get_accesspoint(self, callb=None):
         """Convenience method to request the access point available
 
