@@ -454,6 +454,48 @@ def unpack_lifx_message(packed_message):
             target_addr, source_id, seq_num, payload, ack_requested, response_requested
         )
 
+    elif message_type == MSG_IDS[MultiZoneStateExtendedColorZones]:  # 512
+        zones_count = struct.unpack("H", payload_str[0:2])[0]
+        zone_index = struct.unpack("H", payload_str[2:4])[0]
+        colors_count = struct.unpack("B", payload_str[4:5])[0]
+        colors = []
+        for i in range(82):
+            color = struct.unpack("H" * 4, payload_str[5 + (i * 8) : 13 + (i * 8)])
+            colors.append(color)
+
+        payload = {
+            "zones_count": zones_count,
+            "zone_index": zone_index,
+            "colors_count": colors_count,
+            "colors": colors,
+        }
+        message = MultiZoneStateExtendedColorZones(
+            target_addr, source_id, seq_num, payload, ack_requested, response_requested
+        )
+
+    elif message_type == MSG_IDS[TileStateTileEffect]:  # 720
+        instanceid = struct.unpack("I", payload_str[1:5])[0]
+        effect = struct.unpack("B", payload_str[5:6])[0]
+        speed = struct.unpack("I", payload_str[6:10])[0]
+        duration = struct.unpack("Q", payload_str[10:18])[0]
+        palette_count = struct.unpack("B", payload_str[58:59])[0]
+        palette = []
+        for i in range(16):
+            color = struct.unpack("H" * 4, payload_str[59 + (i * 8) : 67 + (i * 8)])
+            palette.append(color)
+
+        payload = {
+            "instanceid": instanceid,
+            "effect": effect,
+            "speed": speed,
+            "duration": duration,
+            "palette_count": palette_count,
+            "palette": palette,
+        }
+        message = TileStateTileEffect(
+            target_addr, source_id, seq_num, payload, ack_requested, response_requested
+        )
+
     else:
         message = Message(
             message_type,
